@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2011-2017 by Andrew Mustun. All rights reserved.
+ * Copyright (c) 2011-2018 by Andrew Mustun. All rights reserved.
  * 
  * This file is part of the QCAD project.
  *
@@ -17,16 +17,17 @@
  * along with QCAD.
  */
 
-include("../File.js");
+include("scripts/File/File.js");
 include("../Save/Save.js");
 include("scripts/Widgets/ViewportWidget/ViewportWidget.js");
 include("../AutoSave/AutoSave.js");
+include("scripts/Reset/Reset.js");
 
-if (new QFileInfo("scripts/DefaultAction.js").exists()) {
+if (exists("scripts/DefaultAction.js")) {
     include("scripts/DefaultAction.js");
 }
 
-if (new QFileInfo("scripts/Navigation/DefaultNavigation.js").exists()) {
+if (exists("scripts/Navigation/DefaultNavigation.js")) {
     include("scripts/Navigation/DefaultNavigation.js");
 }
 
@@ -50,6 +51,7 @@ NewFile.includeBasePath = includeBasePath;
 
 // list of actions to trigger after opening file:
 NewFile.postOpenActions = [];
+NewFile.postNewActions = [];
 
 NewFile.getScriptFileName = function(scriptFile) {
     if (scriptFile.startsWith(":/")) {
@@ -78,6 +80,26 @@ NewFile.removePostOpenAction = function(scriptFile) {
     NewFile.postOpenActions.remove(sf);
 };
 
+/**
+ * Adds the given script file as action to be triggered after a new document
+ * has been created.
+ */
+NewFile.addPostNewAction = function(scriptFile) {
+    var sf = NewFile.getScriptFileName(scriptFile);
+    if (!NewFile.postNewActions.contains(sf)) {
+        NewFile.postNewActions.push(sf);
+    }
+};
+
+/**
+ * Removes the given script file as action to be triggered after a new document
+ * has been created.
+ */
+NewFile.removePostNewAction = function(scriptFile) {
+    var sf = NewFile.getScriptFileName(scriptFile)
+    NewFile.postNewActions.remove(sf);
+};
+
 /**
  * \param isOpen True if new is called from a file open function.
  */
@@ -207,7 +229,7 @@ NewFile.createMdiChild = function(fileName, nameFilter, uiFile, graphicsSceneCla
     }
 
     var mdiChild = new RMdiChildQt();
-    mdiChild.windowIcon = new QIcon("scripts/qcad_icon.png");
+    mdiChild.windowIcon = new QIcon(autoPath("scripts/qcad_icon.png"));
     mdiChild.setDocumentInterface(documentInterface);
     var flags = new Qt.WindowFlags(Qt.FramelessWindowHint);
     mdiChild.setWindowFlags(flags);
@@ -223,32 +245,10 @@ NewFile.createMdiChild = function(fileName, nameFilter, uiFile, graphicsSceneCla
 
     var viewports = ViewportWidget.getViewports(mdiChild, documentInterface);
     mdiChild.viewports = viewports;
-    //qDebug("initViewports");
     ViewportWidget.initializeViewports(viewports, uiFile, graphicsSceneClass);
-    //qDebug("initViewports: done");
     NewFile.updateTitle(mdiChild);
 
-    // set up default action:
-    var defaultGuiAction = RGuiAction.getByScriptFile("scripts/Reset/Reset.js");
-    var defaultActionFile = RSettings.getStringValue("NewFile/DefaultAction", "");
-    var defaultActionFileInfo = new QFileInfo(defaultActionFile);
-    if (!defaultActionFileInfo.exists()) {
-        defaultActionFileInfo = new QFileInfo(":/" + defaultActionFile);
-    }
-    var defaultAction = undefined;
-    if (defaultActionFile.length>0 && defaultActionFileInfo.exists()) {
-        include(defaultActionFile);
-        var defaultActionClass = new QFileInfo(defaultActionFile).baseName();
-        if (defaultActionFile.length>0 && typeof(global[defaultActionClass])!=="undefined") {
-            defaultAction = new global[defaultActionClass](defaultGuiAction);
-        }
-    }
-    if (isNull(defaultAction) && typeof(DefaultAction)!=="undefined") {
-        defaultAction = new DefaultAction(defaultGuiAction);
-    }
-    if (!isNull(defaultAction)) {
-        documentInterface.setDefaultAction(defaultAction);
-    }
+    NewFile.setupDefaultAction(documentInterface);
 
     ViewportWidget.initEventHandler(viewports);
 
@@ -262,14 +262,14 @@ NewFile.createMdiChild = function(fileName, nameFilter, uiFile, graphicsSceneCla
 
     // Qt 5 / Unity bug workaround:
     // breaks Ubuntu Unity menu on start:
-    if (RS.getSystemId()!=="linux" || !RSettings.isQt(5)) {
-        appWin.enabled = false;
+    if (RS.getSystemId()!=="linux" || !RSettings.isQt(5) || appWin.property("starting")!==true) {
+        appWin.disable();
     }
     for (var i=0; i<5; i++) {
         QCoreApplication.processEvents();
     }
-    if (RS.getSystemId()!=="linux" || !RSettings.isQt(5)) {
-        appWin.enabled = true;
+    if (!appWin.enabled) {
+        appWin.enable();
     }
 
     //qDebug("subWindowActivated");
@@ -280,13 +280,29 @@ NewFile.createMdiChild = function(fileName, nameFilter, uiFile, graphicsSceneCla
         ViewportWidget.updateViewports(viewports);
     }
 
+    var k, action;
     if (isOpen) {
         // trigger post open actions (actions configured by the user to be
         // run whenever a file is opened, for example to show a font
         // replacement dialog, etc):
-        for (var k=0; k<NewFile.postOpenActions.length; k++) {
+        for (k=0; k<NewFile.postOpenActions.length; k++) {
             if (!isNull(NewFile.postOpenActions[k])) {
-                var action = RGuiAction.getByScriptFile(NewFile.postOpenActions[k]);
+                action = RGuiAction.getByScriptFile(NewFile.postOpenActions[k]);
+                if (!isNull(action)) {
+                    action.slotTrigger();
+                }
+            }
+        }
+    }
+    else {
+        // trigger post new action (e.g. to load template file):
+        for (k=0; k<NewFile.postNewActions.length; k++) {
+            if (!isNull(NewFile.postNewActions[k])) {
+                include(NewFile.postNewActions[k]);
+                if (isFunction(initNewFile)) {
+                    initNewFile(mdiChild);
+                }
+                action = RGuiAction.getByScriptFile(NewFile.postNewActions[k]);
                 if (!isNull(action)) {
                     action.slotTrigger();
                 }
@@ -400,6 +416,9 @@ NewFile.closeRequested = function(mdiChild) {
     var discardButton = buttonBox.button(QDialogButtonBox.Discard);
     discardButton.autoDefault = false;
     discardButton['default'] = false;
+    // this would allow the user to not save by pressing the space bar
+    // seems dangerous though
+    //discardButton.setFocus();
 
     var saveButton = buttonBox.button(QDialogButtonBox.Save);
     saveButton.autoDefault = true;
@@ -446,3 +465,54 @@ NewFile.closeRequested = function(mdiChild) {
     EAction.activateMainWindow();
 };
 
+/**
+ * Setup default action for the given document interface.
+ */
+NewFile.setupDefaultAction = function(documentInterface) {
+    var defaultAction = NewFile.getDefaultAction();
+    if (!isNull(defaultAction)) {
+        documentInterface.setDefaultAction(defaultAction);
+    }
+};
+
+/**
+ * Returns name of default action class.
+ */
+NewFile.getDefaultActionClass = function() {
+    var defaultActionClass = "DefaultAction";
+    var defaultActionFile = RSettings.getStringValue("NewFile/DefaultAction", "");
+    if (defaultActionFile.length===0) {
+        return defaultActionClass;
+    }
+    var defaultActionFileInfo = new QFileInfo(defaultActionFile);
+    if (!defaultActionFileInfo.exists()) {
+        defaultActionFileInfo = new QFileInfo(":/" + defaultActionFile);
+    }
+    if (defaultActionFileInfo.exists()) {
+        include(defaultActionFile);
+        defaultActionClass = new QFileInfo(defaultActionFile).baseName();
+    }
+    return defaultActionClass;
+};
+
+/**
+ * Returns new default action as configured in settings ("NewFile/DefaultAction")
+ * or instance of DefaultAction.
+ */
+NewFile.getDefaultAction = function(useGuiAction) {
+    if (isNull(useGuiAction)) {
+        useGuiAction = true;
+    }
+
+    // set up default action:
+    var defaultGuiAction = undefined;
+    if (useGuiAction) {
+        defaultGuiAction = RGuiAction.getByScriptFile("scripts/Reset/Reset.js");
+    }
+    var defaultActionClass = NewFile.getDefaultActionClass();
+    var defaultAction = undefined;
+    if (typeof(global[defaultActionClass])!=="undefined") {
+        defaultAction = new global[defaultActionClass](defaultGuiAction);
+    }
+    return defaultAction;
+};
